package cs451.net.handler;

import cs451.net.event.NetEvent;
import cs451.net.event.NetEventType;
import cs451.parser.Host;
import cs451.parser.Parser;
import cs451.tools.Logger;

public interface NetEventHandlerInterface {


    default void sendIf(){}
    default void deliverIf(){}
    default void beat(){}

    void deliverNextSync(NetEvent ne);
    void sendNextSync(NetEvent ne);
    void crashNextSync(NetEvent ne);

    void deliverNextAsync(NetEvent ne);
    void sendNextAsync(NetEvent ne);
    void crashNextAsync(NetEvent ne);

    default void onEvent(NetEventType et, NetEvent ne){
        switch (et){
            case DLVR: deliver(ne);
                break;
            case SEND: send(ne);
                break;
            case CRSH: crash(ne);
                break;
            case RCVR:
            default:
                Logger.error("Unhandled EventType:"+et);
        }
    }

    default void send(NetEvent ne){
        sendNextSync(ne);
    }
    default void deliver(NetEvent ne){
        deliverNextSync(ne);
    }
    default void crash(NetEvent ne){
        crashNextSync(ne);
    }

    void start(Host h, Parser p);
    void stop();

    default boolean isDone(){ return true;}
}
